var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { nameof } from "ts-simple-nameof";
import { Api } from "../../Api";
import { DataFrame, Method, Status } from "../../DataFrame";
import { ExalusConnectionService } from "../ExalusConnectionService";
import { FieldChangeResult, FieldChangeResultType } from "../FieldChangeResult";
import { SessionService } from "../Session/SessionService";
import { WebApiCacheService } from "../WebApi/WebApiCacheService";
import { ChannelsGroup } from "./IChannelsGroupsService";
export class ChannelsGroupsService {
    constructor() {
        this._serviceName = nameof(ChannelsGroupsService);
        this._connection = null;
        this._synchronized = false;
        this._synchronizationTaskCompletionSource = null;
        this._groups = [];
        this._connection = Api.Get(ExalusConnectionService);
        this._synchronizationTaskCompletionSource = new Promise((resolve) => {
            Api.Get(SessionService).OnUserLoggedInEvent().Subscribe((user) => __awaiter(this, void 0, void 0, function* () {
                this._groups = yield this.GetGroupsAsync();
                resolve();
                this._synchronized = true;
            }));
        });
    }
    GetServiceName() {
        return this._serviceName;
    }
    WaitForSynchronizationAsync() {
        return this._synchronizationTaskCompletionSource;
    }
    GetGroupByGuid(guid) {
        return this._groups.first(a => a.Guid === guid);
    }
    ChangeGroupNameAsync(group, name) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            let req = new AddUpdateChannelsGroupsRequest();
            let gr = new ApiGroupObject();
            gr.Guid = group.Guid;
            gr.Name = name;
            (_a = req.Data) === null || _a === void 0 ? void 0 : _a.push(gr);
            const result = yield ((_b = this._connection) === null || _b === void 0 ? void 0 : _b.SendAndWaitForResponseAsync(req, 15000, true));
            Api.Get(WebApiCacheService).ClearCache();
            switch (result) {
                case Status.OK:
                    group.Name = name;
                    return new FieldChangeResult(FieldChangeResultType.Changed, "");
                case Status.NoPermissionToPerformThisOperation:
                case Status.NoPermissionsToCallGivenResource:
                case Status.UserIsNotLoggedIn:
                    return new FieldChangeResult(FieldChangeResultType.NoPermissions, "");
                case Status.WrongData:
                    return new FieldChangeResult(FieldChangeResultType.WrongData, req.Data);
                default:
                    return new FieldChangeResult(FieldChangeResultType.Failed, "");
            }
        });
    }
    RemoveGroupAsync(group) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            let req = new RemoveGroupRequest();
            req.Data = group.Guid;
            const result = yield ((_a = this._connection) === null || _a === void 0 ? void 0 : _a.SendAndWaitForResponseAsync(req, 15000, true));
            Api.Get(WebApiCacheService).ClearCache();
            this._groups = yield this.GetGroupsAsync();
            if (result != null && result.Status != null)
                return result === null || result === void 0 ? void 0 : result.Status;
            return Status.Error;
        });
    }
    GetGroupsAsync() {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield ((_a = this._connection) === null || _a === void 0 ? void 0 : _a.SendAndWaitForResponseAsync(new GetGroupsRequest(), 15000, true));
            let groups = [];
            switch (result === null || result === void 0 ? void 0 : result.Status) {
                case Status.OK:
                    (_b = result.Data) === null || _b === void 0 ? void 0 : _b.forEach(group => {
                        let gr = new ChannelsGroup();
                        gr.Guid = group.Guid;
                        gr.Name = group.Name;
                        groups.push(gr);
                    });
                    return groups;
                default:
                    throw new Error(`Failed to get groups, API responded with status ${result === null || result === void 0 ? void 0 : result.Status} data: ${result === null || result === void 0 ? void 0 : result.Data}`);
            }
        });
    }
    AddDeviceChannelToGroupAsync(channel, group) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            const req = new AddDeviceChannelToGroupRequest();
            req.Data.Channel = channel.Number;
            let device = channel.GetDevice();
            if (device == null)
                throw new Error("Device is not set on this channel!");
            req.Data.DeviceGuid = (_a = channel.GetDevice()) === null || _a === void 0 ? void 0 : _a.Guid;
            req.Data.GroupGuid = group.Guid;
            const result = yield ((_b = this._connection) === null || _b === void 0 ? void 0 : _b.SendAndWaitForResponseAsync(req, 15000, true));
            Api.Get(WebApiCacheService).ClearCache();
            if (result != null && result.Status != null) {
                if (result.Status == Status.OK)
                    channel.ChannelGroups.push(group.Guid);
                return result.Status;
            }
            return Status.Error;
        });
    }
    RemoveDeviceChannelFromGroupAsync(channel, group) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            const req = new RemoveDeviceChannelToGroupRequest();
            let device = channel.GetDevice();
            if (device == null)
                throw new Error("Device is not set on this channel!");
            req.Data.DeviceGuid = (_a = channel.GetDevice()) === null || _a === void 0 ? void 0 : _a.Guid;
            req.Data.Channel = channel.Number;
            req.Data.GroupGuid = group.Guid;
            const result = yield ((_b = this._connection) === null || _b === void 0 ? void 0 : _b.SendAndWaitForResponseAsync(req, 15000, true));
            Api.Get(WebApiCacheService).ClearCache();
            if (result != null && result.Status != null) {
                if (result.Status == Status.OK)
                    channel.ChannelGroups = channel.ChannelGroups.where(a => a != group.Guid).toArray();
                return result.Status;
            }
            return Status.Error;
        });
    }
    AddNewGroupAsync(group) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            let req = new AddUpdateChannelsGroupsRequest();
            let gr = new ApiGroupObject();
            gr.Name = group.Name;
            (_a = req.Data) === null || _a === void 0 ? void 0 : _a.push(gr);
            const result = yield ((_b = this._connection) === null || _b === void 0 ? void 0 : _b.SendAndWaitForResponseAsync(req, 15000, true));
            Api.Get(WebApiCacheService).ClearCache();
            this._groups = yield this.GetGroupsAsync();
            if (result != null && result.Status != null)
                return result === null || result === void 0 ? void 0 : result.Status;
            return Status.Error;
        });
    }
}
class RemoveGroupRequest extends DataFrame {
    constructor() {
        super();
        this.Resource = "/devices/groups/group";
        this.Method = Method.Delete;
    }
}
class GetGroupsRequest extends DataFrame {
    constructor() {
        super();
        this.Resource = "/devices/groups/";
        this.Method = Method.Get;
    }
}
class AddUpdateChannelsGroupsRequest extends DataFrame {
    constructor() {
        super();
        this.Resource = "/devices/groups/group";
        this.Method = Method.Put;
        this.Data = [];
    }
}
class AddDeviceChannelToGroupRequest extends DataFrame {
    constructor() {
        super();
        this.Resource = "/devices/device/channel/groups/group/";
        this.Method = Method.Put;
        this.Data = new AddRemoveDeviceChannelToGroup();
    }
}
class RemoveDeviceChannelToGroupRequest extends DataFrame {
    constructor() {
        super();
        this.Resource = "/devices/device/channel/groups/group/";
        this.Method = Method.Delete;
        this.Data = new AddRemoveDeviceChannelToGroup();
    }
}
class ApiGroupObject {
    constructor() {
        this.Id = 0;
        this.Guid = "";
        this.Name = "";
    }
}
class AddRemoveDeviceChannelToGroup {
    constructor() {
        this.GroupGuid = "";
        this.DeviceGuid = "";
        this.Channel = 0;
    }
}
//# sourceMappingURL=ChannelsGroupsService.js.map